//
//  ServiceProvider.m
//  UnmountService
//
//  Created by Hugh Cole-Baker on 28/12/2009.
//  Copyright 2009 Hugh Cole-Baker. All rights reserved.
//

#import "ServiceProvider.h"


@implementation ServiceProvider

void appendError(NSString** error, NSString* toAppend)
{
  if((*error) == nil) (*error) = toAppend;
  else (*error) = [(*error) stringByAppendingFormat:@"\n%@",toAppend];
  NSLog(@"%@",toAppend);
}

-(void)unmountVolume:(NSPasteboard *)pboard
            userData:(id)data
               error:(NSString **)error
{
  NSArray *types;
  
	types = [pboard types];
  //Hopefully we have been passed filename(s)
  if([types containsObject:NSFilenamesPboardType])
  {
    NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
    // Perform unmount operation using the list of files
    for(NSString* file in files)
    {
      OSStatus err;
      FSRef ref;
      //Attempt to make an FSRef from the path string
      err = FSPathMakeRef([file fileSystemRepresentation], &ref, NULL);
      if(noErr == err)
      {
        //Try to get a FSVolumeRefNum for this FSRef
        FSCatalogInfo catalogInfo;
        err = FSGetCatalogInfo(&ref,
                               kFSCatInfoVolume,
                               &catalogInfo,
                               NULL,
                               NULL,
                               NULL);
        if(noErr == err)
        {
          FSVolumeRefNum thisVolumeRefNum = catalogInfo.volume;
          pid_t dissenter = 0;
          //Try to unmount the volume, synchronously
          err = FSUnmountVolumeSync(thisVolumeRefNum, 0, &dissenter);
          //Check for errors (such as a process holding open files on the volume)
          if(err != noErr)
          {
            if(dissenter > 0)
            {
              appendError(error,[NSString stringWithFormat:@"PID %d is blocking unmount of %@",dissenter,file]);
            }
            else
            {
              appendError(error,[NSString stringWithFormat:@"Could not unmount %@",file]);
            }
          }
        }
        else
        {
          appendError(error,[NSString stringWithFormat:@"Could not get filesystem catalog info for volume %@", file]);
        }
      }
      else
      {
        appendError(error,[NSString stringWithFormat:@"Could not get filesystem reference for volume %@", file]);
      }
    }
  }
}

@end
